package jalview.gui;

import java.awt.*;
import javax.swing.*;
import jalview.datamodel.*;
import jalview.analysis.*;


public class SeqCanvas extends JPanel
{
     FeatureRenderer fr;
    Image             img;
    Graphics          gg;
    int               imgWidth;
    int               imgHeight;

    AlignViewport     av;

    public boolean paintFlag = false;

    boolean showScores = false;
    boolean displaySearch = false;
    int [] searchResults = null;

    int chunkHeight;
    int chunkWidth;


    public SeqCanvas(AlignViewport av)
    {
	this.av         = av;
       fr = new FeatureRenderer(av);
       setLayout(new BorderLayout());
       PaintRefresher.Register(this);

    }

  public void drawScale(Graphics g, int startx, int endx,int ypos) {
      int scalestartx = startx - startx%10 + 10;

      g.setColor(Color.black);

      for (int i=scalestartx;i < endx;i+= 10)
      {
          String string = String.valueOf(i);
          g.drawString(string,(i-startx-1)*av.charWidth,ypos - av.charHeight/2);

          g.drawLine( (i-startx-1)*av.charWidth +av.charWidth/2, ypos+2 - av.charHeight/2,
                    (i-startx-1)*av.charWidth +av.charWidth/2, ypos-2 );

      }

  }


/**
 * Definitions of startx and endx (hopefully):
 * SMJS This is what I'm working towards!
 *   startx is the first residue (starting at 0) to display.
 *   endx   is the last residue to display (starting at 0).
 *   starty is the first sequence to display (starting at 0).
 *   endy   is the last sequence to display (starting at 0).
 * NOTE 1: The av limits are set in setFont in this class and
 * in the adjustment listener in SeqPanel when the scrollbars move.
 */

  public void paintComponent(Graphics g)
  {

    if (img==null ||  imgWidth!=getWidth() || imgHeight!=getHeight() || paintFlag)
    {
      imgWidth  = getWidth();
      imgHeight = getHeight();
      img = createImage(imgWidth,imgHeight);
      gg  = img.getGraphics();
      gg.setFont(av.getFont());
      paintFlag = false;
    }


    chunkWidth  =   getWidth()/av.charWidth;
    chunkHeight =  (av.getAlignment().getHeight() + 2)*av.charHeight;

    av.setChunkHeight(chunkHeight);
    av.setChunkWidth(chunkWidth);

    fillBackground(gg,Color.WHITE,0,0,imgWidth,imgHeight);

    if (av.getWrapAlignment())
      drawWrappedPanel(gg, getWidth(), getHeight(), av.startRes);
    else
      drawPanel(gg, av.startRes, av.endRes, av.startSeq, av.endSeq, av.startRes, av.startSeq, 0);


    g.drawImage(img,0,0,this);

  }

  public void drawWrappedPanel(Graphics g, int canvasWidth, int canvasHeight, int startRes)
  {
      AlignmentI da = av.getAlignment();

      int cWidth  =   canvasWidth/av.charWidth;
      int cHeight =  (av.getAlignment().getHeight() + 2)*av.charHeight;

      int  endx   = startRes+cWidth-1;
      int ypos  = 2*av.charHeight;

      while (ypos <= canvasHeight)
      {
        drawScale(g, startRes, endx, ypos);
        drawPanel(g, startRes, endx, 0, da.getHeight(), startRes, 0, ypos);

        ypos += cHeight;
        startRes += cWidth;
        endx = startRes + cWidth - 1;

        if (endx > da.getWidth())
          endx = da.getWidth();
      }

  }


  public void drawPanel(Graphics g,int x1,int x2, int y1, int y2,int startx, int starty,int offset) {


    g.setFont(av.getFont());
    RendererI sr = av.getRenderer();

    /*Vector    pid    = av.getConsensus(false);
    Vector tmpseq = new Vector();
    for (int i = 0; i < av.getAlignment().getHeight(); i++)
	if (!av.getSelection().contains(av.getAlignment().getSequenceAt(i)))
	    tmpseq.addElement(av.getAlignment().getSequenceAt(i));

    if (sr instanceof SequenceRenderer)
	pid    = AAFrequency.calculate(tmpseq,x1,x2);

    else if (sr instanceof GraphRenderer)
	pid = AAFrequency.calculatePID(av.getAlignment().getSequenceAt(0),
				       av.getAlignment().getSequences(),
				       av.getPIDWindow(),x1,x2);

*/

   /* if (y2 > starty && y1 < av.getEndSeq())
    {
       fillBackground(g,
                   Color.red,
                   (x1-startx)*charWidth,
                   offset + AlignmentUtil.getPixelHeight(starty,y1,av.getCharHeight()),
                   (x2-x1+1)*charWidth,
                   offset + AlignmentUtil.getPixelHeight(y1,y2,av.getCharHeight()));
    }*/

    SequenceI nextSeq;

    /// First draw the sequences
    /////////////////////////////
    for (int i = y1 ; i < y2 ;i++)
    {
     nextSeq = av.getAlignment().getSequenceAt(i);

     sr.drawSequence(g, nextSeq, av.alignment.findAllGroups( nextSeq ),x1,x2,
                 (x1 - startx) * av.charWidth,
                 offset + AlignmentUtil.getPixelHeight(starty, i, av.charHeight),
                 av.charWidth,av.charHeight,null, i);

     if(av.showSequenceFeatures)
     {
       fr.drawSequence(g, nextSeq, av.alignment.findAllGroups( nextSeq ), x1, x2,
                       (x1 - startx) * av.charWidth,
                       offset +
                       AlignmentUtil.getPixelHeight(starty, i, av.charHeight),
                       av.charWidth, av.charHeight, null, i);
     }
    }
    //
    /////////////////////////////////////

    // Now outline any areas if necessary
    /////////////////////////////////////
    SequenceGroup group = av.getSelectionGroup();
    java.util.Vector groups = av.alignment.getGroups();

    int sx = -1, sy = -1, ex = -1;
    int groupIndex = -1;
    if (group == null && groups.size() > 0)
    {
      group = (SequenceGroup) groups.elementAt(0);
      groupIndex = 0;
    }

    if (group != null)
      do
      {
        int oldY = -1;
        int i = 0;
        boolean inGroup = false;
        for (i = y1; i < y2; i++)
        {
          sx = (group.getStartRes() - startx) * av.charWidth;
          sy = offset + AlignmentUtil.getPixelHeight(starty, i, av.charHeight);
          ex = (group.getEndRes() + 1 - group.getStartRes()) * av.charWidth;

          if (group.sequences.contains(av.alignment.getSequenceAt(i))
              && sx < getWidth()
              && ex > 0)
          {
            if (!inGroup)
            {
              oldY = sy;
              inGroup = true;
              if (group == av.getSelectionGroup())
                g.setColor(Color.RED.brighter());
              else
                g.setColor(group.getOutlineColour());
            }
          }
          else
          {
            if (inGroup)
            {
              g.drawRect(sx, oldY, ex, (sy - oldY));
              inGroup = false;
            }
          }
        }
        if (inGroup)
        {
          sy = offset + AlignmentUtil.getPixelHeight(starty, i, av.charHeight);
          g.drawRect(sx, oldY, ex, (sy - oldY));
          inGroup = false;
        }

        groupIndex++;
        if (groupIndex >= groups.size())
          break;

        group = (SequenceGroup) groups.elementAt(groupIndex);

      }
      while (groupIndex < groups.size());


    /// Highlight search Results once all sequences have been drawn
    //////////////////////////////////////////////////////////
    if(displaySearch)
    {
      for(int r=0; r<searchResults.length; r+=3)
      {
        int searchSeq = searchResults[r];
        int searchStart = searchResults[r+1];
        int searchEnd = searchResults[r+2];

        if (searchSeq >= y1 && searchSeq < y2)
        {
          SequenceRenderer ssr = (SequenceRenderer) sr;
          ssr.drawHighlightedText(av.getAlignment().getSequenceAt(searchSeq),
                                  searchStart,
                                  searchEnd,
                                  (searchStart - startx) * av.charWidth,
                                  offset +
                                  AlignmentUtil.getPixelHeight(starty, searchSeq,
              av.charHeight),
                                  av.charWidth,
                                  av.charHeight);
        }
      }
    }

  }

  public void fillBackground(Graphics g,Color c, int x1,int y1,int width,int height) {
    g.setColor(c);
    g.fillRect(x1,y1,width,height);
  }

  public int getChunkWidth() {
    return chunkWidth;
  }

  public void highlightSearchResults(int [] results)
  {
    // results are in the order sequence, startRes, endRes
    if(results==null)
      displaySearch = false;
    else
      displaySearch = true;

    searchResults = results;

    paintFlag = true;
    repaint();
  }


}
